from tcsoa.gen.Internal_RevRuleMgmt._2019_12.services import RevisionRuleAdministrationService as imp0
from tcsoa.utils import TcService


class RevisionRuleAdministrationService(TcService):
    createOrUpdateRevisionRule = imp0.createOrUpdateRevisionRule
    getRevisionRuleInfo = imp0.getRevisionRuleInfo
