from tcsoa.gen.Translation._2007_06.services import TranslationManagementService as imp0
from tcsoa.utils import TcService


class TranslationManagementService(TcService):
    createTranslationRequest = imp0.createTranslationRequest
