from tcsoa.gen.Internal_ConfigFilterCriteria._2013_12.services import VariantManagementService as imp0
from tcsoa.utils import TcService


class VariantManagementService(TcService):
    executeConfiguratorService = imp0.executeConfiguratorService
