from tcsoa.gen.ActiveWorkspaceBom._2015_10.services import OccurrenceManagementCadService as imp0
from tcsoa.utils import TcService


class OccurrenceManagementCadService(TcService):
    createBOMWindowsFromBookmarks = imp0.createBOMWindowsFromBookmarks
    createBOMWindowsFromContexts = imp0.createBOMWindowsFromContexts
